#!/bin/csh
#
# Data Assimilation Research Testbed -- DART
# Copyright 2004-2007, Data Assimilation Research Section
# University Corporation for Atmospheric Research
# Licensed under the GPL -- www.gpl.org/licenses/gpl.html
#
# <next few lines under version control, do not edit>
# $URL: http://subversion.ucar.edu/DAReS/DART/trunk/models/ikeda/work/workshop_setup.csh $
# $Id: workshop_setup.csh 2752 2007-03-30 22:12:18Z thoar $
# $Revision: 2752 $
# $Date: 2007-03-30 16:12:18 -0600 (Fri, 30 Mar 2007) $

# Initial conditions were generated by observing state 
# variable 1 with an enormous ( ~1,000,000.0) error variance.
# The observation was defined to be taken at day=0 seconds=0.
# fixed_network_sequence was run to create a sequence with 3000 
# hourly observations starting at day=0 seconds=0.
# The initial conditions for filter can accomodate 100 ensemble members.
# TJH 39-Mar-2007

#----------------------------------------------------------------------
# 'preprocess' is a program that culls the appropriate sections of the
# observation module for the observations types in 'input.nml'; the 
# resulting source file is used by all the remaining programs, 
# so this MUST be run first.
#----------------------------------------------------------------------

\rm -f preprocess *.o *.mod
\rm -f ../../../obs_def/obs_def_mod.f90
\rm -f ../../../obs_kind/obs_kind_mod.f90

set MODEL = "ikeda"

@ n = 1

echo
echo
echo "---------------------------------------------------------------"
echo "${MODEL} build number ${n} is preprocess"

csh  mkmf_preprocess
make || exit $n

./preprocess || exit 99

#----------------------------------------------------------------------
# Build all the single-threaded targets
#----------------------------------------------------------------------

foreach TARGET ( mkmf_* )

   set PROG = `echo $TARGET | sed -e 's#mkmf_##'`

   switch ( $TARGET )
   case mkmf_preprocess:
      breaksw
   default:
      @ n = $n + 1
      echo
      echo "---------------------------------------------------"
      echo "${MODEL} build number ${n} is ${PROG}" 
      \rm -f ${PROG}
      csh $TARGET || exit $n
      make        || exit $n
      breaksw
   endsw
end

@ n = $n + 1
./perfect_model_obs || exit $n

@ n = $n + 1
./filter            || exit $n

